package example;

import java.net.SocketException;

import coap.*;
import endpoint.*;
import example.SampleClient;

/*
 * This class implements 
 *    CoAP client API
 *    CoAP server with simple resources to print any received msessages
 *    The UpdÖayer has a thread which listens to incoming messages
 */
public class CoAPClientServer extends LocalEndpoint {

	/*
	 * Constructor for a new CoAPClientServer
	 * 
     */
	public CoAPClientServer() throws SocketException {		

		// add default resource to the server
		addResource(new DefaultDisplayResource());
		System.out.printf("CoAPClientServer listening at port %d.\n", this.port());
		// add further resources using addResource() before launching the server thread
	}
	
	// Resource definitions ////////////////////////////////////////////////////
	private class DefaultDisplayResource extends LocalResource {
		
		public DefaultDisplayResource() {
			super("DefaultDisplayResource");
			setResourceName("POST or PUT to display on console");
		}
				
		
		@Override
		public void performPOST(POSTRequest request) {

			System.out.println(request.getPayloadString()); // Display the string
			
			// complete the request
			request.respond(CodeRegistry.RESP_CHANGED);
		}
		
		@Override
		public void performPUT(PUTRequest request) {

			System.out.println(request.getPayloadString()); // Display the string
			
			// complete the request
			request.respond(CodeRegistry.RESP_CHANGED);
		}		
		
	}

	

	// Logging /////////////////////////////////////////////////////////////////
	
	@Override
	public void handleRequest(Request request) {
		
		// output the request
		System.out.println("Incoming request:");
		request.log();
		
		// handle the request
		super.handleRequest(request);
	}
	
	// This is the CoAP Client API for sending messages
	public static void sendCoapMsg(String[] args) {
		
		SampleClient.main(args);
		
	} // end of sendCoapMsg
	
} // End of class CoAPClientServer
