package com.ericsson.p2psip.m2m;


import java.io.Serializable;
import java.util.Vector;
import com.ericsson.p2psip.p2ppparser.EncodedSequence;

public class Book implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 4528160140392687842L;
	public static final int MAX_ProxyNodeS = 100;
	public static final int L_NUMBEROFProxyNodeS = 4;

	private int npns;
	private Vector<ProxyNode> pns;


	public Book() {
		this.npns = 0;
		this.pns = new Vector<ProxyNode>();
	}


	public Book(Book another) {
		this.npns = another.npns;
		this.pns = new Vector<ProxyNode>();
		for(int i=0; i<another.pns.size(); i++) {
			this.pns.addElement(another.pns.elementAt(i));
		}
	}


	public static Book parse(EncodedSequence encodedSequence) {
		Book book = new Book();
		if(encodedSequence.getBytes().length != 0) {
			book.setNpns(encodedSequence.parseInt());
			Vector<ProxyNode> pns =  new Vector<ProxyNode>();
			for(int i=0; i<book.npns; i++) {
				ProxyNode entry = ProxyNode.parse(encodedSequence);
				pns.addElement(entry);
			}					
			book.setPns(pns);
		}	
		return book;
	}

	public byte[] getBytes() {
		EncodedSequence encodedSequence = new EncodedSequence(this.getLength());
		if(encodedSequence.getBytes().length != 0) {
			encodedSequence.addInt(this.npns);

			for (int i=0; i < this.npns; i++ ){
				ProxyNode entry = (ProxyNode) pns.elementAt(i);
				entry.encode(encodedSequence);
			}
		}
		return encodedSequence.getBytes();
	}

	public byte[] encode(EncodedSequence encodedSequence){
		
		if(encodedSequence.getBytes().length != 0) {
			encodedSequence.addInt(this.npns);

			for (int i=0; i < this.npns; i++ ){
				ProxyNode entry = (ProxyNode) pns.elementAt(i);
				entry.encode(encodedSequence);
			}
		}
		return encodedSequence.getBytes();
	}

	public int getLength() {
		int length = 4;
		for (int i=0; i < this.pns.size(); i++ ){
			ProxyNode entry = (ProxyNode) pns.elementAt(i);
			length += entry.getLength();
		}
		
		return length;		
	}

	public int getNpns() {
		return npns;
	}

	public void setNpns(int npns) {
		this.npns = npns;
	}

	public Vector<ProxyNode> getPns() {
		return pns;
	}

	public void setPns(Vector<ProxyNode> pns) {
		this.pns = pns;
	}

	public void insertAtEndPN(ProxyNode pninfo) {
		this.pns.addElement(pninfo);
		this.setNpns(npns + 1);
	}
	
	public void setPN(ProxyNode pninfo, int pos) {
		this.pns.setElementAt(pninfo, pos);
	}

	public void updatePN(ProxyNode ProxyNode1, int i) {
		this.pns.set(i, ProxyNode1);	
	}

	public void print() {
		System.out.println ("Book {");
		
		for (int i=0; i < this.pns.size(); i++ ){
			ProxyNode entry = (ProxyNode) pns.elementAt(i);
			entry.print();		
		}
		
		System.out.println ("}");
	}

	public int findPN(ProxyNode pnode) {
		int result = -1;
		for (int i = 0; i < this.pns.size(); i++) {
			ProxyNode entry = (ProxyNode) this.pns.elementAt(i);
			if (entry.getPn().getName().equals(pnode.getPn().getName())) {
				result = i;
			}
		}
		return result;
	}
	
}