package com.ericsson.p2psip.m2m;

import java.util.Arrays;

import com.ericsson.p2psip.p2ppparser.EncodedSequence;
import com.ericsson.p2psip.util.IPEndpoint;

import junit.framework.TestCase;


public class BookkeepingTest extends TestCase {

	public IP2P4M2M engine;

	public Book book;
	//private P2pSipLogger logger;
	
	public void testEncodeAndDecode(){
	
		/**
		 * TEST IPEndPoint
		 */
		
		// CREATE
		IPEndpoint ipa = new IPEndpoint("127.0.0.1", 4564);
		System.out.println("ipa:" + ipa.toString());
	
		
		// ENCODE
		EncodedSequence esIP = new EncodedSequence(20);
		ipa.encode(esIP);
		System.out.println("ipa bytes:" + Arrays.toString(esIP.getBytes()));
		
		// PARSE
		esIP.setPos(0);
		IPEndpoint ipa2 = IPEndpoint.parse(esIP);
		System.out.println("ipa2:" + ipa2.toString());
		
		/**
		 * TEST RECORD
		 */
		
		// CREATE
		long t_join =123;
		long t_leave = System.currentTimeMillis();
		Record r1 = new Record(3,"peter",new IPEndpoint("127.0.0.1", 342) , t_join, t_leave);
		System.out.println("Record 1:" + r1.toString());
		
		// ENCODE
		EncodedSequence encodedSequence = new EncodedSequence(Record.LENGTH);
		r1.encode(encodedSequence);
		
		// PARSE
		encodedSequence.setPos(0);
		Record r2 = Record.parse(encodedSequence);
		System.out.println("Record 2:" + r2.toString());
		
		/**
		 * TEST PROXY NODE
		 */
		
		//CREATE
		ProxyNode n1 = new ProxyNode();
		n1.setPn(r1);
		Record ln1 = new Record(0,"adolf",new IPEndpoint("127.0.0.1", 342) , t_join, t_leave);
		Record ln2 = new Record(1,"ben",new IPEndpoint("127.0.0.1", 342) , t_join, t_leave);
		Record ln3 = new Record(2,"carl",new IPEndpoint("127.0.0.1", 342) , t_join, t_leave);
		Record ln4 = new Record(3,"dimitry",new IPEndpoint("127.0.0.1", 342) , t_join, t_leave);
		
		//INSERT
		n1.addLN(ln1);
		n1.addLN(ln2);
		n1.addLN(ln3);
		n1.addLN(ln4);
		
		n1.print();
		
		
		//ENCODE
		EncodedSequence espn = 
			new EncodedSequence(n1.getLength());
		System.out.println("length "+ n1.getLength());
		System.out.println("local nodes "+ n1.getLns().size());
		n1.encode(espn);
		
		//PARSE
		espn.setPos(0);
		ProxyNode n2 = ProxyNode.parse(espn);
		n2.print();	
	
		/**
		 * TEST BOOK
		 */
		
		//CREATE
		Book b1 = new Book();
		
		//INSERT
		for (int i = 0; i < 4; i++) {
			ProxyNode anode = new ProxyNode(n1);
			anode.setPn(new Record(i,"test"+i,new IPEndpoint("127.0.0.1", 342) , t_join, t_leave));
			b1.insertAtEndPN(anode);
		}
		b1.print();
		
		//ENCODE
		EncodedSequence esB1 = 
			new EncodedSequence(b1.getLength());
		b1.encode(esB1);
		
		//PARSE
		esB1.setPos(0);
		Book b2 = Book.parse(esB1);
		
		b2.print();	

		//ENCODE
		EncodedSequence esB3 = 
			new EncodedSequence(b1.getBytes());
		
		
		//PARSE
		esB3.setPos(0);
		Book b3 = Book.parse(esB3);
		
		b3.print();	
		
		
		/**
		 * TEST UPDATE LIST
		 */
		
		//CREATE
		UpdateList ul1 = new UpdateList();
		
		//INSERT
		for (int i = 0; i < 4; i++) {
			NameIP ni = new NameIP("test"+i,new IPEndpoint("127.0.0.1", i),(byte)1);
			ul1.addnameip(ni);
		}
		ul1.print();
		
		//ENCODE
		EncodedSequence esUL1 = 
			new EncodedSequence(ul1.getLength());
		ul1.encode(esUL1);
		
		//PARSE
		esUL1.setPos(0);
		UpdateList ul2 = UpdateList.parse(esUL1);
		
		ul2.print();
		
		
	}

}


