package com.ericsson.p2psip.m2m;

import java.rmi.Remote;
import java.rmi.RemoteException;

import com.ericsson.p2psip.util.IPEndpoint;

public interface IM2M extends Remote{

	/**
	 * Retrieves a resource stored in the DHT overlay
	 * @param key String to be looked for.
	 * @return byte[] with the stored value.
	 */
	public abstract byte[] get(String key) throws RemoteException;

	/**
	 * Stores information in the DHT overlay.
	 * @param key identifier used for the lookup.
	 * @param value information to be stored.
	 */
	public abstract void put(String key, byte[] value) throws RemoteException;

	/**
	 * Joining Method. It is different for the Nodes that are part
	 * of the overlay and those who are not (LNs).
	 * @param coapName Name of the joining node. For WN and PN the same as in the configuration file.
	 * @param indht represents whether the node is part of the DHT (True) or not (False).
	 */
	public abstract void join(String coapName, boolean indht) throws RemoteException;

	/**
	 * The method detaches a Node from the overlay.
	 * It is assumed that the coapName will be either of a local WN, PN or LN, not a network node.
	 * @param coapName name of the leaving node. 
	 */
	public abstract void leave(String coapName) throws RemoteException;

	/** 
	 * Returns the IP address as IPEndPoint stored in the DHT.
	 * @param key The name to be searched for in the DHT. 
	 */
	public abstract IPEndpoint getIP(String key) throws RemoteException;

	/** 
	 * Returns a complete list of all the nodes in the DHT.
	 */
	public abstract Book getList() throws RemoteException;

	/** 
	 * Returns the list of updates in the DHT.
	 * It erases the list each time it is called.
	 */
	public abstract UpdateList getUpdates() throws RemoteException;
	
	/** 
	 * Returns Bookkeeping Data from the node running the DHT.
	 */
	public Bookkeeping getBookData() throws RemoteException;
}