package com.ericsson.p2psip.m2m;

/**
 * An API to a P2P overlay.
 * @author ejoumae
 *
 */
public interface IP2P4M2M {

	/**
	 * Join the P2P overlay.
	 */
	public void asyncJoin();
	
	/**
	 * Leave the P2P overlay.
	 */
	public void asyncLeave();
		
	/**
	 * Perform a lookup in the P2P overlay. The result of the lookup will be
	 * returned in a callback (please see the interface IP2P4M2MCallback).
	 * @param lookupID is an identifier that will be returned to the caller in
	 * the callback.
	 * @param key is the key that is to be searched for from the overlay.
	 */
	public void asyncLookup(String lookupID, String key);

	/**
	 * Perform a lookup in the P2P overlay. The result of the lookup will be
	 * returned in a callback (please see the interface IP2P4M2MCallback).
	 * @param lookupID is an identifier that will be returned to the caller in
	 * the callback.
	 * @param key is the key that is to be searched for from the overlay.
	 */
	public void asyncLookup(String lookupID, byte[] key);

	
	/**
	 * Store data (i.e., a <key, value> pair) in the P2P overlay.
	 * @param storeID is an identifier that will be returned to the caller in
	 * the callback.
	 * @param key is the key of the data in the overlay.
	 * @param value is the value that is to be stored in the overlay.
	 */
	public void asyncStore(String storeID, String key, byte[] value);
	
	/**
	 * Store data (i.e., a <key, value> pair) in the P2P overlay.
	 * @param storeID is an identifier that will be returned to the caller in
	 * the callback.
	 * @param key is the key of the data in the overlay.
	 * @param value is the value that is to be stored in the overlay.
	 */
	public void asyncStore(String storeID, byte[] key, byte[] value);
	
	/**
	 * Remove data associated with the specified key from the overlay.
	 * @param removeID is an identifier that will be returned to the caller in
	 * the callback.
	 * @param key is the key of the record that is to be removed.
	 */
	public void asyncRemove(String removeID, String key);
	
	/**
	 * Remove data associated with the specified key from the overlay.
	 * @param removeID is an identifier that will be returned to the caller in
	 * the callback.
	 * @param key is the key of the record that is to be removed.
	 */
	public void asyncRemove(String removeID, byte[] key);

//	
//	/**
//	 * Store data (i.e., a <key, value> pair) in the P2P overlay.
//	 * @param storeID is an identifier that will be returned to the caller in
//	 * the callback.
//	 * @param key is the key of the data in the overlay.
//	 * @param value is the value that is to be stored in the overlay.
//	 */
//	public void asyncLookupM2M(String lookup_shortinfo_join, String shortinfo);

}
