package com.ericsson.p2psip.m2m;

/**
 * A callback interface that is used by P2P4M2MEngine to deliver callbacks
 * @author ejoumae
 *
 */
public interface IP2P4M2MCallback {
	
	/**
	 * This method will be called by P2P4M2MEngine when the join operation has
	 * finished.
	 * @param result contains the result of the join operation
	 */
	public void joinReady(P2P4M2MCallbackResult result);
	
	/**
	 * This method will be called by P2P4M2MEngine when the leave operation has
	 * finished.
	 * @param result contains the result of the leave operation
	 */
	public void leaveReady(P2P4M2MCallbackResult result);
	
	/**
	 * This method will be called by the P2P4M2MEngine when a lookup operation
	 * has finished.
	 * @param result contains the result of the lookup operation.
	 */
	public void lookupReady(P2P4M2MLookupResult result);
	
	/**
	 * This method will be called by the P2P4M2MEngine when a store operation
	 * has finished.
	 * @param storeID can be used by the receiver of the callback to map this 
	 * the callback to a call to IP2P4M2M.asyncStore( )
	 * @param result contains the result of the store operation
	 */
	public void storeReady(String storeID, P2P4M2MCallbackResult result);
	
	/**
	 * This method will be called by the P2P4M2MEngine when a remove operation
	 * has finished.
	 * @param removeID can be used by the receiver of the callback to map this 
	 * the callback to a call to IP2P4M2M.asyncRemove( )
	 * @param result contains the result of the remove operation
	 */
	public void removeReady(String removeID, P2P4M2MCallbackResult result);
	
}
