package com.ericsson.p2psip.m2m;

import java.io.IOException;
import java.net.URI;
import java.util.Arrays;
import java.net.URISyntaxException;

import org.bouncycastle.util.Strings;

import com.ericsson.coap.coap.*;
import com.ericsson.coap.example.SampleClient;

import com.ericsson.p2psip.util.IPEndpoint;
import com.ericsson.p2psip.m2m.P2P4M2MCE;


public class M2MExample {
	
	
	public static void coapmain(String[] args) {

		// initialize parameters
		String method  = null;
		String uri     = null;
		String payload = null;
		boolean loop   = false;

		// display help if no parameters specified
		if (args.length == 0) {
			SampleClient.printInfo();
			return;
		}

		// input parameters
		int idx = 0;
		for (String arg : args) {
			if (arg.startsWith("-")) {
				if (arg.equals("-l")) {
					loop = true;
				} else {
					System.out.println("Unrecognized option: " + arg);
				}
			} else {
				switch (idx) {
				case SampleClient.IDX_METHOD:	
					method = arg.toUpperCase();
					break;
				case SampleClient.IDX_URI:	
					uri = arg;
					break;
				case SampleClient.IDX_PAYLOAD: 
					payload = arg;
					break;
				default:
					System.out.println("Unexpected argument: " + arg);
				}
				++idx;
			}
		}
			
		// create request according to specified method 
		if (method == null) {
			System.err.println("Method not specified");
			return;
		}
		Request request = SampleClient.newRequest(method);
		if (request == null) {
			System.err.println("Unknown method: " + method);
			return;
		}
		
		if (method.equals("OBSERVE")) {
			request.setOption(new Option(60, OptionNumberRegistry.OBSERVE));
			loop = true;
		}
		
		// set request URI
		if (uri == null) {
			System.err.println("URI not specified");
		}
		if (method.equals("DISCOVER") && !uri.endsWith(SampleClient.DISCOVERY_RESOURCE)) {
			uri = uri + SampleClient.DISCOVERY_RESOURCE;
		}
		try {
			request.setURI(new URI(uri));
		} catch (URISyntaxException e) {
			System.err.println("Failed to parse URI: " + e.getMessage());
			return;
		}
		// set request payload
		request.setPayload(payload);
		
		// enable response queue in order to use blocking I/O
		request.enableResponseQueue(true);
		
		// execute request
		try {
			request.execute();
		} catch (IOException e) {
			System.err.println("Failed to execute request: " + e.getMessage());
			return;
		}

		// loop for receiving multiple responses
		do {
		
			// receive response
			
			System.out.println("Receiving response...");
			Response response = null;
			try {
				response = request.receiveResponse();
				
				// check for indirect response
				if (response != null && response.isEmptyACK()) {
					response.log();
					System.out.println("Request acknowledged, waiting for separate response...");
					
					response = request.receiveResponse();
				}
				
			} catch (InterruptedException e) {
				System.err.println("Failed to receive response: " + e.getMessage());
				return;
			}
	
			// output response
			
			if (response != null) {
				
				response.log();
				System.out.println("Round Trip Time (ms): " + response.getRTT());
			
				// check of response contains resources
				if (response.hasFormat(MediaTypeRegistry.LINK_FORMAT)) {
					
					String linkFormat = response.getPayloadString();
					
					// create resource three from link format
					Resource root = RemoteResource.newRoot(linkFormat);
					if (root != null) {
						
						// output discovered resources
						System.out.println("\nDiscovered resources:");
						root.log();
						
					} else {
						System.err.println("Failed to parse link format");
					}
				} else {
					
					// check if link format was expected by client
					if (method.equals("DISCOVER")) {
						System.out.println("Server error: Link format not specified");
					}
				}
				
			} else {
				
				// no response received
				// calculate time elapsed 
				long elapsed = System.currentTimeMillis() - request.getTimestamp();
				
				System.out.println("Request timed out (ms): " + elapsed);
				break;
			}
			
		} while (loop);
		
		// finish
		System.out.println();
	}

	
	
	public static void main (String[] args) throws InterruptedException {
		/* Start the nodes*/
		if(args.length < 1) {
			System.err.println("Wrong number of arguments. Usage: java -jar "
					+ "m2mce.jar -c config_file.txt ");
			System.exit(1);
		}
		int i = 0;
		if (args[i++].equals("-c")) {
			P2P4M2MCE.CONFIG_FILE = args[i].toString();
		}
		else {
			System.err.println("ERROR: -c is missing");
			return;
		}	
//		
//		
//		IM2M ce = new P2P4M2MCE();
//
//		System.out.println("Starting test");
//		
//		/* A PN or WN joins*/
//		String name = ce.getBookData().getPnode().getPn().getName();
//		ce.join(name, true);
//		Thread.sleep(1000);
//		
//		/* LNs join*/
//		ce.join("ln1", false);
//		Thread.sleep(1000);
//		ce.join("ln2", false);
//		Thread.sleep(1000);
//		ce.join("ln3", false);
//		Thread.sleep(1000);
//		ce.join("ln4", false);
//		Thread.sleep(1000);
//		
//		UpdateList ul1 = ce.getUpdates();
//		System.out.println("Printing Update List: ");
//		ul1.print();
//		Thread.sleep(1000);
//		Book b1 = ce.getList();
//		System.out.println("Printing Book: ");
//		b1.print();
//		Thread.sleep(1000);
//		IPEndpoint ip = ce.getIP("node0");
//		System.out.println("node0: " + ip.toString());	
//
//		IPEndpoint ip2 = ce.getIP("ln3");
//		System.out.println("ln3: " + ip2.toString());
//		
//		byte[] value = new byte[]{(byte)10};
//		ce.put("Example", value);
//		System.out.println("Put: " + Arrays.toString(value));
//		
//		byte[] value2 = ce.get("Example");
//		System.out.println("Get: " + Arrays.toString(value2));
//		
//		/* LNs leave*/
//		ce.leave("ln2");
//		Thread.sleep(1000);
//		ce.leave("ln1");
//		Thread.sleep(1000);
//		
//		
//		UpdateList ul2 = ce.getUpdates();
//		System.out.println("Printing Update List: ");
//		ul2.print();
//		Thread.sleep(1000);
//		Book b2 = ce.getList();
//		System.out.println("Printing Book: ");
//		b2.print();
//
//		/* LNs leave*/
//		ce.leave("ln3");
//		Thread.sleep(1000);
//		ce.join("ln1", false);
//		Thread.sleep(1000);
//		
//		UpdateList ul3 = ce.getUpdates();
//		System.out.println("Printing Update List: ");
//		ul3.print();
//		Thread.sleep(1000);
//		Book b3 = ce.getList();
//		System.out.println("Printing Book: ");
//		b3.print();
//		

		String coap1 = "GET";
		String coap2 = "coap://localhost/.well-known/core";
		
		String[] coapinput = {coap1, coap2};
		
		
		coapmain(coapinput);
	}
	
	
	
}
