package com.ericsson.p2psip.m2m;


import java.io.Serializable;

import com.ericsson.p2psip.p2ppparser.EncodedSequence;
import com.ericsson.p2psip.util.IPEndpoint;


public class NameIP implements Serializable{

	/**
	 * 
	 */
	private static final long serialVersionUID = 4407976046281961604L;
	public static final int L_NAME = 32;
	public static final int L_IP = 16;
	public static final int L_PORT = 4;
	public static final int L_STATUS = 1;
	public static final int LENGTH = L_NAME + L_IP  + L_PORT + L_STATUS; 


	private String name;
	private IPEndpoint ip;
	private byte status;

	/**
	 * Constructs a new empty Book.
	 */
	public NameIP() {
		this.name = "";
		this.ip = new IPEndpoint();
		this.status = 0;
	}

	public NameIP(NameIP another) {
		this.name = another.name;
		this.ip = another.ip;
		this.status = another.status;
	}
	
	/**
	 * 
	 * @param name
	 * @param ip
	 * @param status 1 if it is in overlay, 0 if it is not.
	 */
	public NameIP(String name, IPEndpoint ip, byte status) {
		this.name = name;
		this.ip = ip;
		this.status = status;
	}
	

	public void encode(EncodedSequence encodedSequence) {
		byte[] namebytes = new byte[NameIP.L_NAME];
		System.arraycopy(name.getBytes(), 0, namebytes, 0, name.getBytes().length);
		encodedSequence.add(namebytes);
		this.ip.encode(encodedSequence);
		encodedSequence.addByte(status);
	}

	
	public static NameIP parse(EncodedSequence encodedSequence) {
		NameIP NameIP = new NameIP();
		NameIP.setName(new String(encodedSequence.parseByteArray(L_NAME)).trim());
		IPEndpoint ip = IPEndpoint.parse(encodedSequence);
		NameIP.setIp(ip);
		NameIP.setStatus(encodedSequence.parseByte());
		return NameIP;
	}
	
	public byte getStatus() {
		return status;
	}

	public void setStatus(byte status) {
		this.status = status;
	}

	public IPEndpoint getIp() {
		return ip;
	}

	public void setIp(IPEndpoint ip) {
		this.ip = ip;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	
	public byte[] getBytes() {
		EncodedSequence eS = new EncodedSequence(NameIP.LENGTH);
		byte[] namebytes = new byte[NameIP.L_NAME];
		System.arraycopy(name.getBytes(), 0, namebytes, 0, name.getBytes().length);
		eS.add(namebytes);
		this.ip.encode(eS);
		eS.addByte(status);
		return eS.getBytes();
	}
	
	@Override
	public String toString() {
		
		return "[" +name + " | " + ip.getIpAddress().toString() + ":" + ip.getPort() + " | " + this.status + "]";
	}
	
}