package com.ericsson.p2psip.m2m;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Date;

import com.ericsson.p2psip.logger.P2pSipLogger;
import com.ericsson.p2psip.p2ppparser.EncodedSequence;
import com.ericsson.p2psip.util.ConfigData;
import com.ericsson.p2psip.util.IPEndpoint;

// TODO: TEST SIMPLE EXAMPLES joining of PN, LN, leaving, getting LISTS... etc.

public class P2P4M2MCE extends UnicastRemoteObject implements IP2P4M2MCallback,
		IM2M {
	/**
	 * The configuration file
	 */
	public static String CONFIG_FILE;
	/**
	 * The logger
	 */
	private P2pSipLogger logger;
	/**
	 * The P2P4M2MEngine
	 */
	private IP2P4M2M engine;
	/**
	 * For storing bookkeeping data
	 */
	public Bookkeeping bookData;
	/**
	 * For storing lookups
	 */
	private P2P4M2MLookUps lu;
	/**
	 * Name and IP of the node
	 */
	private NameIP nip;
	/**
	 * Indicates whether this peer has joined the overlay or not
	 */
	private boolean joined;

	private boolean resultIsReady;

	private boolean isPN = false;

	private P2P4M2MLookupResult lu_result;

	private static byte[] o = new byte[4];

	/**
	 * Creates the P2P4M2M instance. If the node is a Bootstrap peer it also
	 * initializes the bookkeeping information.
	 */
	public P2P4M2MCE() throws RemoteException {

		this.joined = false;
		this.logger = P2pSipLogger.getInstance("P2P4M2MCE");
		this.engine = new P2P4M2MEngine(this);
		P2P4M2MEngine temp = (P2P4M2MEngine) this.engine;

		this.lu = new P2P4M2MLookUps();
		this.bookData = new Bookkeeping();
		Record pninfo = new Record(0, temp.getName(), temp.getIPEndpoint(),
				(new Date()).getTime(), 0);
		this.bookData.setPnode(new ProxyNode(pninfo));

		logger.logToConsole("P2P4M2MCE: Local IP is "
				+ temp.getIPEndpoint().toString() + ".", P2pSipLogger.INFO);
		logger.logToConsole("P2P4M2MCE: Local name is " + temp.getName() + ".",
				P2pSipLogger.INFO);

		if (temp.getConfig().getBooleanProperty(ConfigData.BOOTSTRAP_MODE)) {
			this.joined = true;
			NameIP anip = new NameIP(temp.getName(), temp.getIPEndpoint(),
					(byte) 0);
			UpdateList aul = new UpdateList();
			aul.addnameip(anip);
			this.bookData.setUlist(aul);

			/**
			 * INITIALIZE BOOKKEEPING and Bootstrap Peer info
			 */

			logger.logToConsole("P2P4M2MCE: Node is Bootstrap ",
					P2pSipLogger.INFO);
			// Store Short Generic Information
			engine.asyncStore((new Long((new Date()).getTime())).toString(),
					M2MCEKeys.SHORTINFO, new ShortInfo().getBytes());
			// Store Book
			engine.asyncStore((new Long((new Date()).getTime())).toString(),
					M2MCEKeys.LASTBOOK, P2P4M2MCE.o);
			// Store Update List
			engine.asyncStore((new Long((new Date()).getTime())).toString(),
					M2MCEKeys.UPDATELIST, P2P4M2MCE.o);
			// Store Name-IP of BP
			engine.asyncStore((new Long((new Date()).getTime())).toString(),
					this.bookData.getPnode().getPn().getName(), this.bookData
							.getPnode().getPn().getIp().getIpPortBytes20());

			logger.logToConsole("P2P4M2MCE: Bookkeeping values initialized.",
					P2pSipLogger.INFO);
		}

	}

	/************* LOOKUP RESULTS *************/

	public void lookupReady(P2P4M2MLookupResult result) {

		/************************************************
		 * Normal Operations
		 ************************************************/
		logger.logToConsole(
				"P2P4M2MCE.lookupReady Result is ready, store result.",
				P2pSipLogger.INFO);
		synchronized (this) {
			this.resultIsReady = true;
			this.lu_result = result;
		}
		printLookUpResult(result);

		/************************************************
		 * Bookkeeping Operations
		 ************************************************/
		String lookup = result.getLookupID();
		if (result.getStatus() == P2P4M2MLookupResult.SUCCESS) {

			/************ WHEN A NODE IS JOINING ***********/

			/** Short Info Join */
			if (lookup.equals(lu.getLookup_shortinfo_join())) {
				/* If the node is JOINING modifies SHORT_INFO */
				logger.logToConsole(
						"P2P4M2MCE.lookupReady Node has joined, modifying shortinfo.",
						P2pSipLogger.INFO);
				ShortInfo info = ShortInfo.parse(new EncodedSequence(result
						.getData()));
				int added = info.getAdded_nodes() + 1;
				info.setAdded_nodes(added);
				this.bookData.setShortinfo(info);

				engine.asyncStore(
						(new Long((new Date()).getTime())).toString(),
						M2MCEKeys.SHORTINFO, info.getBytes());
				// info.print();
			}

			/** Book Join */
			else if (lookup.equals(lu.getLookup_last_book_join())) {
				/* If the node is JOINING get LAST_BOOK */
				logger.logToConsole(
						"P2P4M2MCE.lookupReady Node has joined, modifying last book.",
						P2pSipLogger.INFO);
				Book book = new Book();
				if (result.getData() != P2P4M2MCE.o) {
					book = Book.parse(new EncodedSequence(result.getData()));
				}
				this.bookData.setMybook(book);
				// this.bookData.getPnode().getPn().setVectorpos(book.getPns().size());

				int findposition = book.findPN(this.bookData.getPnode());
				if (findposition >= 0) {
					book.setPN(this.bookData.getPnode(), findposition);
				} else {
					book.insertAtEndPN(this.bookData.getPnode());
				}

				engine.asyncStore(
						(new Long((new Date()).getTime())).toString(),
						M2MCEKeys.LASTBOOK, book.getBytes());

				// book.print();
			}

			/** Update List Join */
			else if (lookup.equals(lu.getLookup_update_list_join())) {
				/* If the node is JOINING get UPDATE_LIST */
				logger.logToConsole(
						"P2P4M2MCE.lookupReady Node has joined, modifying updatelist.",
						P2pSipLogger.INFO);
				UpdateList ul = new UpdateList();
				if (result.getData() != P2P4M2MCE.o) {
					ul = UpdateList
							.parse(new EncodedSequence(result.getData()));
				}
				ul.addnameip(nip);
				this.bookData.setUlist(ul);
				engine.asyncStore(
						(new Long((new Date()).getTime())).toString(),
						M2MCEKeys.UPDATELIST, ul.getBytes());

				// ul.print();
			}

			/************ WHEN A NODE IS LEAVING ***********/

			else if ((lookup.equals(lu.getLookup_name_leave()))
					|| (lookup.equals(lu.getLookup_update_list_leave()))
					|| (lookup.equals(lu.getLookup_last_book_leave()))) {
				/** Update List Leave */
				if (lookup.equals(lu.getLookup_update_list_leave())) {
					/* If the node is LEAVING get UPDATE_LIST */
					logger.logToConsole(
							"P2P4M2MCE.lookupReady Node is leaving, modifying updatelist.",
							P2pSipLogger.INFO);
					UpdateList ul = new UpdateList();
					if (result.getData().length != 0) {
						ul = UpdateList.parse(new EncodedSequence(result
								.getData()));
					}
					ul.addnameip(nip);
					/*
					 * If the node leaving is a PN, then also the dependent LNs
					 * will leave
					 */
					if (nip.getName().equals(
							this.bookData.getPnode().getPn().getName())) {
						for (int i = 0; i < this.bookData.getPnode().getNlns(); i++) {
							Record aln = this.bookData.getPnode().getLns()
									.elementAt(i);
							ul.addnameip(new NameIP(aln.getName(), aln.getIp(),
									(byte) 0));
						}
					}
					this.bookData.setUlist(ul);
					engine.asyncStore(
							(new Long((new Date()).getTime())).toString(),
							M2MCEKeys.UPDATELIST, ul.getBytes());
					// ul.print();
				}
				/** Book Leave */
				else if (lookup.equals(lu.getLookup_last_book_leave())) {
					/* If the node is LEAVING get LAST_BOOK */
					logger.logToConsole(
							"P2P4M2MCE.lookupReady Node is leaving, modifying last book.",
							P2pSipLogger.INFO);
					Book book = new Book();
					if (result.getData().length != 0) {
						book = Book
								.parse(new EncodedSequence(result.getData()));
					}

					int findposition = book.findPN(this.bookData.getPnode());
					book.setPN(this.bookData.getPnode(), findposition);
					engine.asyncStore(
							(new Long((new Date()).getTime())).toString(),
							M2MCEKeys.LASTBOOK, book.getBytes());

					// book.print();
				}
				/** Short Info Leave */
				else if (lookup.equals(lu.getLookup_shortinfo_leave())) {
					/* If the node is LEAVING modifies SHORT_INFO */
					logger.logToConsole(
							"P2P4M2MCE.lookupReady Node is leaving, modifying short info.",
							P2pSipLogger.INFO);
					ShortInfo info = ShortInfo.parse(new EncodedSequence(result
							.getData()));
					int departed = info.getDeparted_nodes() + 1;
					info.setDeparted_nodes(departed);
					this.bookData.setShortinfo(info);

					engine.asyncStore(
							(new Long((new Date()).getTime())).toString(),
							M2MCEKeys.SHORTINFO, info.getBytes());
					// info.print();
				}
				if (isPN) {
					isPN = false;
					engine.asyncLeave();
				}
			}

			else {
				logger.logToConsole("LookupReady LookUpID is Unknown: "
						+ result.toString(), P2pSipLogger.INFO);
			}
		} else if (result.getStatus() == P2P4M2MLookupResult.FAILURE) {
			logger.logToConsole(
					"LookupReady results was a FAILURE: " + result.toString(),
					P2pSipLogger.INFO);
		}

		else {
			logger.logToConsole(
					"LookupReady results is Strange: " + result.toString(),
					P2pSipLogger.INFO);
		}
	}

	private void printLookUpResult(P2P4M2MLookupResult result) {
		// logger.logToConsole("Lookup results received: " + result.toString(),
		// P2pSipLogger.INFO);
		//
		// if(result.getData() != P2P4M2MCE.o) {
		// logger.logToConsole("Value: " + result.getData().toString(),
		// P2pSipLogger.INFO);
		// logger.logToConsole("Lookup results received: " +
		// BitOperations.intBytesToString(result.getData()),
		// P2pSipLogger.INFO);
		// }
	}

	/**
	 * @inheritDoc
	 */
	public void storeReady(String storeID, P2P4M2MCallbackResult result) {
		logger.logToConsole("P2P4M2MCE.storeReady( ): result: " + result,
				P2pSipLogger.INFO);
	}

	/**
	 * @inheritDoc
	 */
	public void removeReady(String removeID, P2P4M2MCallbackResult result) {
		logger.logToConsole("P2P4M2MCE.removeReady( ): result: " + result,
				P2pSipLogger.INFO);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ericsson.p2psip.m2m.IM2M#get(java.lang.String)
	 */
	@Override
	public byte[] get(String key) {

		if (!joined) {
			System.out.println("Please join first!");
		}
		try {
			logger.logToConsole("P2P4M2MCE.get(String).", P2pSipLogger.INFO);
			this.lu.setLookup_getString((new Long((new Date()).getTime()))
					.toString());
			engine.asyncLookup(this.lu.getLookup_getString(), key);
			while (true) {
				if ((this.resultIsReady)
						&& (lu_result.getLookupID().equals(lu
								.getLookup_getString()))) {

					if ((lu_result.getStatus() == P2P4M2MLookupResult.SUCCESS)) {
						this.resultIsReady = false;
					} else {
						System.out
								.println("P2P4M2MCE.getIP: The Lookup failed");
					}
					break;
				}
				Thread.sleep(100);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return lu_result.getData();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ericsson.p2psip.m2m.IM2M#put(java.lang.String, byte[])
	 */
	@Override
	public void put(String key, byte[] value) {
		if (!joined) {
			System.out.println("Please join first!");
			return;
		}
		engine.asyncStore((new Long((new Date()).getTime())).toString(), key,
				value);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ericsson.p2psip.m2m.IM2M#join(java.lang.String, boolean)
	 */
	@Override
	public void join(String coapName, boolean indht) {
		logger.logToConsole("P2P4M2MCE.join.", P2pSipLogger.INFO);
		this.nip = new NameIP(coapName, this.bookData.getPnode().getPn()
				.getIp(), (byte) 1);
		if (indht) {
			logger.logToConsole("P2P4M2MCE.join Node is in DHT.",
					P2pSipLogger.INFO);
			engine.asyncJoin();
			this.bookData.getPnode().getPn().setName(coapName);
		} else {
			ProxyNode pn = this.bookData.getPnode();
			boolean there = false;
			for (int i = 0; i < pn.getNlns(); i++) {
				if (coapName.equals(pn.getLns().elementAt(i).getName())) {
					System.out.println("Old Local Node.");
					pn.getLns().elementAt(i).setT_join(new Date().getTime());
					pn.getLns().elementAt(i).setT_leave(0);
					there = true;
				}
			}
			this.bookData.setPnode(pn);
			if (!there) {
				System.out.println("New Local Node. ");
				Record ln = new Record(pn.getNlns(), coapName, pn.getPn()
						.getIp(), (new Date()).getTime(), 0);
				pn.addLN(ln);
			}

			this.bookData.setPnode(pn);
			// Insert CoAP Name and Address.
			engine.asyncStore((new Long((new Date()).getTime())).toString(),
					nip.getName(), this.bookData.getPnode().getPn().getIp()
							.getIpPortBytes20());
			joinlookups();
		}
	}

	/**
	 * @inheritDoc
	 */
	public void joinReady(P2P4M2MCallbackResult result) {
		logger.logToConsole("P2P4M2MCE.joinReady( ): result: " + result,
				P2pSipLogger.INFO);
		this.joined = true;

		// Insert CoAP Name and Address.
		engine.asyncStore((new Long((new Date()).getTime())).toString(),
				nip.getName(), this.bookData.getPnode().getPn().getIp()
						.getIpPortBytes20());

		joinlookups();
	}

	private void joinlookups() {

		logger.logToConsole("P2P4M2MCE.joinlookups.", P2pSipLogger.INFO);
		// Look for ShortInfo.
		logger.logToConsole("P2P4M2MCE.joinReady( ): Now Looking for: "
				+ M2MCEKeys.SHORTINFO, 4);
		this.lu.setLookup_shortinfo_join((new Long((new Date()).getTime()))
				.toString());
		engine.asyncLookup(this.lu.getLookup_shortinfo_join(),
				M2MCEKeys.SHORTINFO);

		// Look for Update List
		logger.logToConsole("P2P4M2MCE.joinReady( ): Now Looking for: "
				+ M2MCEKeys.UPDATELIST, 4);
		this.lu.setLookup_update_list_join((new Long((new Date()).getTime()))
				.toString());
		engine.asyncLookup(this.lu.getLookup_update_list_join(),
				M2MCEKeys.UPDATELIST);

		// Look for Last Book
		logger.logToConsole("P2P4M2MCE.joinReady( ): Now Looking for: "
				+ M2MCEKeys.LASTBOOK, 4);
		this.lu.setLookup_last_book_join((new Long((new Date()).getTime()))
				.toString());
		engine.asyncLookup(this.lu.getLookup_last_book_join(),
				M2MCEKeys.LASTBOOK);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ericsson.p2psip.m2m.IM2M#leave(java.lang.String)
	 */
	@Override
	public void leave(String coapName) {
		isPN = false;
		/**
		 * Find and modify stored PN information (both for PN and LNs associated
		 * to PN.
		 */
		this.nip = new NameIP(coapName, this.bookData.getPnode().getPn()
				.getIp(), (byte) 0);

		if (coapName.equals(this.bookData.getPnode().getPn().getName())) {
			isPN = true;
			logger.logToConsole("P2P4M2MCE.leave Node is PN or WN.",
					P2pSipLogger.INFO);
			this.bookData.getPnode().getPn().setT_leave(new Date().getTime());
		} else if (!coapName.equals(this.bookData.getPnode().getPn().getName())) {
			logger.logToConsole("P2P4M2MCE.leave Node is LN.",
					P2pSipLogger.INFO);
			ProxyNode pn = this.bookData.getPnode();

			for (int i = 0; i < pn.getNlns(); i++) {
				if (coapName.equals(pn.getLns().elementAt(i).getName())) {
					pn.getLns().elementAt(i).setT_leave(new Date().getTime());
				}
			}
			this.bookData.setPnode(pn);
		}
		// Remove Name directly.
		engine.asyncRemove((new Long((new Date()).getTime())).toString(),
				coapName);

		leavelookups();
	}

	private void leavelookups() {
		logger.logToConsole("P2P4M2MCE.leavelookups.", P2pSipLogger.INFO);
		// Look for ShortInfo.
		this.lu.setLookup_shortinfo_leave((new Long((new Date()).getTime()))
				.toString());
		engine.asyncLookup(this.lu.getLookup_shortinfo_leave(),
				M2MCEKeys.SHORTINFO);

		// Look for UpdateList
		this.lu.setLookup_update_list_leave((new Long((new Date()).getTime()))
				.toString());
		engine.asyncLookup(this.lu.getLookup_update_list_leave(),
				M2MCEKeys.UPDATELIST);

		// Look for Last Book
		this.lu.setLookup_last_book_leave((new Long((new Date()).getTime()))
				.toString());
		engine.asyncLookup(this.lu.getLookup_last_book_leave(),
				M2MCEKeys.LASTBOOK);
	}

	/**
	 * @inheritDoc
	 */
	public void leaveReady(P2P4M2MCallbackResult result) {
		logger.logToConsole("P2P4M2MCE.leaveReady( ): result: " + result,
				P2pSipLogger.INFO);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ericsson.p2psip.m2m.IM2M#getIP(java.lang.String)
	 */
	@Override
	public IPEndpoint getIP(String key) {
		IPEndpoint ip = new IPEndpoint();
		if (!joined) {
			System.out.println("Please join first!");
		}
		try {
			logger.logToConsole("P2P4M2MCE.getIP.", P2pSipLogger.INFO);
			this.lu.setLookup_getIP((new Long((new Date()).getTime()))
					.toString());
			engine.asyncLookup(this.lu.getLookup_getIP(), key);
			while (true) {
				if ((this.resultIsReady)
						&& (lu_result.getLookupID()
								.equals(lu.getLookup_getIP()))) {

					if ((lu_result.getStatus() == P2P4M2MLookupResult.SUCCESS)) {
						ip = IPEndpoint.parse(new EncodedSequence(
								this.lu_result.getData()));
						this.resultIsReady = false;
					} else {
						System.out
								.println("P2P4M2MCE.getIP: The Lookup failed");
					}
					break;
				}
				Thread.sleep(100);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return ip;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ericsson.p2psip.m2m.IM2M#getList()
	 */
	@Override
	public Book getList() {
		Book book = new Book();
		if (!joined) {
			System.out.println("Please join first!");
		}
		try {
			logger.logToConsole("P2P4M2MCE.getList.", P2pSipLogger.INFO);
			this.lu.setLookup_getList((new Long((new Date()).getTime()))
					.toString());
			engine.asyncLookup(this.lu.getLookup_getList(), M2MCEKeys.LASTBOOK);
			while (true) {
				if ((this.resultIsReady)
						&& (lu_result.getLookupID().equals(lu
								.getLookup_getList()))) {

					if ((lu_result.getStatus() == P2P4M2MLookupResult.SUCCESS)) {
						book = Book.parse(new EncodedSequence(this.lu_result
								.getData()));
						this.resultIsReady = false;
					} else {
						System.out
								.println("P2P4M2MCE.getList: The Lookup failed");
					}
					break;
				}
				Thread.sleep(100);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		// book.print();
		return book;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ericsson.p2psip.m2m.IM2M#getUpdates()
	 */
	@Override
	public UpdateList getUpdates() {
		UpdateList uls = new UpdateList();
		if (!joined) {
			System.out.println("Please join first!");
		}
		try {
			logger.logToConsole("P2P4M2MCE.getUpdates.", P2pSipLogger.INFO);
			this.lu.setLookup_getUpdates((new Long((new Date()).getTime()))
					.toString());
			engine.asyncLookup(this.lu.getLookup_getUpdates(),
					M2MCEKeys.UPDATELIST);
			while (true) {
				if ((this.resultIsReady)
						&& (lu_result.getLookupID().equals(lu
								.getLookup_getUpdates()))) {
					if ((lu_result.getStatus() == P2P4M2MLookupResult.SUCCESS)) {
						if (lu_result.getData().equals(P2P4M2MCE.o)) {
							System.out
									.println("P2P4M2MCE.getUpdates: UpdateList is empty");
						} else {
							uls = UpdateList.parse(new EncodedSequence(
									this.lu_result.getData()));
						}
						this.resultIsReady = false;
					} else {
						System.out
								.println("P2P4M2MCE.getUpdates: The Lookup failed");
					}
					break;
				}
				Thread.sleep(100);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		logger.logToConsole("P2P4M2MCE.getUpdates Storing 0 in List.",
				P2pSipLogger.INFO);

		engine.asyncStore((new Long((new Date()).getTime())).toString(),
				M2MCEKeys.UPDATELIST, P2P4M2MCE.o);

		// uls.print();
		return uls;
	}

	public Bookkeeping getBookData() {
		return bookData;
	}

}
