package com.ericsson.p2psip.m2m;

import java.io.Serializable;

/**
 * A class representing a P2P4M2M lookup result
 * @author ejoumae
 *
 */
public class P2P4M2MLookupResult implements Serializable{
		
	/**
	 * 
	 */
	private static final long serialVersionUID = 8468345669873885936L;
	/**
	 * Contains the data fetched from the overlay
	 */
	private byte[] data;
	/**
	 * Indicates the status of this lookup (e.g., success or failure)
	 */
	private int status;
	/**
	 * Contains the same lookupID that was passed to P2P4M2MEngine.asyncLookup
	 */
	private String lookupID;
	
	/**
	 * The lookup was successful
	 */
	public static final int SUCCESS = 0;
	/**
	 * The lookup failed
	 */
	public static final int FAILURE = 1;
	
	/**
	 * Create a P2P4M2MLookupResult instance
	 * @param status indicates the status of the lookup
	 * @param data contains the record fetched from the overlay
	 * @param lookupID can be used by the received to associate the lookup
	 * result with a call to IP2P4M2M.asyncLookup(lookupID, ...)
	 */
	public P2P4M2MLookupResult(int status, byte[] data, String lookupID) {
		this.data = data;
		this.status = status;
		this.lookupID = lookupID;
	}

	/**
	 * Get the record returned by the lookup
	 * @return
	 */
	public byte[] getData() {
		return data;
	}

	/**
	 * Set the data
	 * @param data
	 */
	public void setData(byte[] data) {
		this.data = data;
	}

	/**
	 * Get the status code indicating the result of the lookup (e.g., success or
	 * failure)
	 * @return
	 */
	public int getStatus() {
		return status;
	}

	/**
	 * Set the status code
	 * @param status
	 */
	public void setStatus(int status) {
		this.status = status;
	}
	
	/**
	 * Get the lookup-ID
	 * @return
	 */
	public String getLookupID() {
		return this.lookupID;
	}	
	
	/**
	 * Convert the status code into a string representation
	 * @param status is a status code
	 * @return
	 */
	public static String statusToStr(int status) {
		String result = "";
		switch(status) {
		case P2P4M2MLookupResult.SUCCESS : result = "success"; break;
		case P2P4M2MLookupResult.FAILURE : result = "failure"; break;
		default : result = "unknown"; break;
		}
		return result;
	}
	
	
	/**
	 * Get a textual representation of this object
	 */
	public String toString() {
		String result = P2P4M2MLookupResult.statusToStr(this.status);
		if(this.data != null) {
			result += ", bytes: " + this.data.length;
		}
		result += ", lookup-ID: " + this.lookupID;
		return result;
	}
	
}
