package com.ericsson.p2psip.m2m;

import com.ericsson.jiam.Configuration;
import com.ericsson.jiam.sip.address.Address;
import com.ericsson.jiam.sip.parser.ParseException;
import com.ericsson.jiam.sip.parser.SipParser;
import com.ericsson.p2psip.P2ppConstants;
import com.ericsson.p2psip.chord.ChordConstants;
import com.ericsson.p2psip.logger.LogWriter;
import com.ericsson.p2psip.util.AppUtils;
import com.ericsson.p2psip.util.ConfigData;
import com.ericsson.p2psip.util.IPEndpoint;
import com.ericsson.p2psip.util.Peer;
import com.ericsson.p2psip.util.UserNameGenerator;

/**
 * Contains various utility methods for P2P4M2MEngine
 * @author ejoumae
 *
 */
public abstract class P2P4M2MUtils {

	
	public static void setConfiguration(ConfigData p2pConfigData) {
		
		/* Read the config file */
		try {
			AppUtils.readConfigFile(P2P4M2MCE.CONFIG_FILE, p2pConfigData, 
					true);
		}
		catch(Exception e) {
			System.out.println("ERROR while reading the config file:");
			e.printStackTrace();
			System.exit(0);
		}
		
		/* Some Chord settings */
		ChordConstants chordSettings = ChordConstants.getInstance();
		chordSettings.setProperty(ChordConstants.SETTING_PERFORM_REPLICATION, 
				false);
		chordSettings.setProperty(ChordConstants.SETTING_USE_CONNECT_REQS, 
				true);
		chordSettings.setProperty(
				ChordConstants.SETTING_USE_PRED_LIST_IN_ET_REQS, false);
		chordSettings.setProperty(ChordConstants.SETTING_USE_PRED_STABILIZATION, 
				true);
		chordSettings.setProperty(
				ChordConstants.SETTING_USE_SUCC_LIST_IN_KA_REQS, false);
		
    	/* Indicates to user name generator whether SIP URIs should be used */
    	p2pConfigData.setProperty(ConfigData.USING_SIP, false);
    	p2pConfigData.setProperty(ConfigData.CONSOLE_OUTPUT_LEVEL, 
    			LogWriter.FULL_OUTPUT);

    	
    	    	
    	/* Get the SIP URI (i.e. the user name) to be used in the P2P overlay */
    	String sipUserAoR = p2pConfigData.getProperty("SIP_USER_AOR");
    	String userName = null;

   		try {
   			Address addr = SipParser.getParser(sipUserAoR).SipAddress();
   			userName = addr.getURI().toString();
   			
   		} 
   		catch(ParseException e) {
   			throw new RuntimeException(e.getMessage());
   		}
   		
   		/* If random user names are used, then clear the user name */
   		if(p2pConfigData.getBooleanProperty("randomUserName") == true) {
   			userName = null;
   
   		}
    	
   		p2pConfigData.setUserNameGenerator(new UserNameGenerator(
   				p2pConfigData));
   		
    	if(userName == null || userName.length() == 0) {
    		if(p2pConfigData.getBooleanProperty(
    				ConfigData.CENTRAL_USERNAME_ASSIGNMENT) 
    				&& p2pConfigData.getBooleanProperty(
    						ConfigData.BOOTSTRAP_MODE)) 
    		{
    			/*
    			 * Give a special name to the user at the bootstrap peer (alice)
    			 */
    			userName = p2pConfigData.getUserNameGenerator().getFirstName(); 
    		}
    		else {
    			userName = AppUtils.generateUniqueUserName(
    					p2pConfigData.getIntProperty(ConfigData.LOCAL_PORT), 
    					p2pConfigData.getBooleanProperty(ConfigData.USING_SIP));
    		}
    	}

   		/* Set SIP configuration */
        Configuration.setProperty("SIP_USER_AOR", userName);
      	Configuration.setProperty("sipServerPortPublic", 
      			p2pConfigData.getProperty("sipServerPortPublic"));
      	Configuration.setProperty("sipServerPortLocal", 
      			p2pConfigData.getProperty("sipServerPortLocal"));
        Configuration.setProperty("sipTimerT1", "500");
        Configuration.setProperty("sipTimerT2", "4000");
        Configuration.setProperty("sipTimerT4", "5000");
    	
    	String transport = p2pConfigData.getProperty(
    			ConfigData.TRANSPORT_PROTOCOL);
    	if(transport.equalsIgnoreCase(P2ppConstants.TRANSPORT_UDP)) {
    		transport = P2ppConstants.TRANSPORT_UDP;
    	}
    	else if(transport.equalsIgnoreCase(P2ppConstants.TRANSPORT_TCP)) {
    		transport = P2ppConstants.TRANSPORT_TCP;
    	}
    	else {
    		transport = P2ppConstants.TRANSPORT_STUN;
    	}
      	p2pConfigData.updateProperty(ConfigData.TRANSPORT_PROTOCOL, transport);
      	
    	/* Bootstrap peer */
        IPEndpoint bootstrapServerAddress = new IPEndpoint(
        		p2pConfigData.getProperty("bootstrapPeerIP"), 
        		p2pConfigData.getIntProperty("bootstrapPeerPort"), transport);
        Peer bootstrapPeer = new Peer(bootstrapServerAddress);
        p2pConfigData.setBootstrapPeer(bootstrapPeer);
	
        /* Local IP */
        int localServerPort = p2pConfigData.getIntProperty(
        		ConfigData.LOCAL_PORT);
        IPEndpoint ipaddress = new IPEndpoint(p2pConfigData.getProperty(
    			ConfigData.LOCAL_IP_STRING), localServerPort, transport);
    	p2pConfigData.setLocalIP(ipaddress);
    	p2pConfigData.setProperty(ConfigData.LOG_FILE, "Logfile_"
    			+ ipaddress.toStringWithoutTransport()+".txt");
    	
//    	p2pConfigData.setProperty(ConfigData.LOG_FILE, "Logfile.txt");
	}
    
	
}
