package com.ericsson.p2psip.m2m;

import java.io.Serializable;
import java.util.Vector;

import com.ericsson.p2psip.p2ppparser.EncodedSequence;


public class ProxyNode implements Serializable{

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 987001126756600075L;
	/**
	 * Node's Bookkeeping information.
	 * Stores Record information of the PN/WN node and its associated LNs.
	 */
	/**
	 * Should contain: 
	 * Record of the Node
	 * Number of LNs
	 * Length of LNs
	 *   Record of the LN1
	 * 	 Record of the LN2
	 * 	 Record of the LN3
	 * 	 Record of the LN4 
	 */
	

	/* The record information of this WN or PN. */
	private Record pn;
	/* Number of local nodes in lns. */
	private int nlns;
	/* The record information of the LN dependent of this PN. */
	private Vector<Record> lns;
		
	/**
	 * Constructs a new empty Proxy Node.
	 */
	public ProxyNode() {
		this.pn = new Record();
		this.nlns = 0;
		this.lns = new Vector<Record>();
	}

	public ProxyNode(Record proxynodeinfo) {
		this.pn = proxynodeinfo;
		this.nlns = 0;
		this.lns = new Vector<Record>();
	}

	public ProxyNode(ProxyNode another) {
		this.pn = another.pn;
		this.nlns = another.nlns;
		this.lns = new Vector<Record>();
		for(int i=0; i<another.lns.size(); i++) {
			this.lns.addElement(another.lns.elementAt(i));
		}
	}

	public static ProxyNode parse(EncodedSequence encodedSequence) {
		ProxyNode n = new ProxyNode();
		n.setPn(Record.parse(encodedSequence));
		n.setNlns(encodedSequence.parseInt());
		Vector<Record> lns =  new Vector<Record>();
		for(int i=0; i<n.getNlns(); i++) {
			Record entry = Record.parse(encodedSequence);
			lns.addElement(entry);
		}						
		n.setLns(lns);

		return n;
	}

	public byte[] encode(EncodedSequence encodedSequence){
		pn.encode(encodedSequence);
		encodedSequence.addInt(this.nlns);
		for (int i=0; i < this.nlns; i++ ){
			Record entry = (Record) lns.elementAt(i);
			entry.encode(encodedSequence);
		}
		return encodedSequence.getBytes();
	}

	public byte[] getBytes() {
		EncodedSequence encodedSequence = new EncodedSequence(this.getLength());
		this.pn.encode(encodedSequence);
		encodedSequence.addInt(this.nlns);
		for (int i=0; i < this.nlns; i++ ){
			Record entry = (Record) lns.elementAt(i);
			entry.encode(encodedSequence);
		}
		return encodedSequence.getBytes();
	}

	public Record getPn() {
		return pn;
	}


	public void setPn(Record pn) {
		this.pn = pn;
	}


	public int getNlns() {
		return nlns;
	}


	public void setNlns(int nlns) {
		this.nlns = nlns;
	}

	public Vector<Record> getLns() {
		return lns;
	}


	public void setLns(Vector<Record> lns) {
		this.lns = lns;
	}

	public int getLength() {
		return Record.LENGTH + 4 + 4 + (this.lns.size()*Record.LENGTH);		
	}

	public void print() {
		System.out.println ("PN: " + this.pn.toString());
		if (nlns >0) {
			System.out.println ("  LNs:");
			for (int i=0; i < this.lns.size(); i++ ){
				Record entry = (Record) lns.elementAt(i);
				System.out.println ("     " + i + entry.toString());	
			}
		}
		//System.out.println ("}");
	}


	public void addLN(Record r) {
		this.lns.add(r);
		this.setNlns(nlns+1);
	}
	

}