package com.ericsson.p2psip.m2m;


import java.io.Serializable;

import com.ericsson.p2psip.p2ppparser.EncodedSequence;
import com.ericsson.p2psip.util.IPEndpoint;


/**
 *  0                   1                   2                   3
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * | name                  
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |        ip                      | t_join        |tLeave |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * 
 */


public class Record implements Serializable{

	/**
	 * 
	 */
	private static final long serialVersionUID = -6334752500624757412L;
	public static final int L_POSITION = 4;
	public static final int L_NAME = 32;
	public static final int L_IP = 16;
	public static final int L_PORT = 4;
	public static final int L_TIMESTAMP = 8;
	public static final int LENGTH = L_POSITION+ L_NAME + L_IP  + L_PORT + L_TIMESTAMP + L_TIMESTAMP; 

	private int vectorpos;
	private String name;
	private IPEndpoint ip;
	private long t_join;
	private long t_leave;

	/**
	 * Constructs a new empty Book.
	 */
	public Record() {
		this.vectorpos = -1;
		this.name = "";
		this.ip = new IPEndpoint();
		this.t_join = -1;
		this.t_leave = -1;
	}

	public Record(Record another) {
		this.vectorpos = another.vectorpos;
		this.name = another.name;
		this.ip = another.ip;
		this.t_join = another.t_join;
		this.t_leave = another.t_leave;
	}
	
	public Record(int vectorpos, String name, IPEndpoint ip,long t_join, long t_leave) {
		this.vectorpos= vectorpos;
		this.name = name;
		this.ip = ip;
		this.t_join = t_join;
		this.t_leave = t_leave;
	}
	

	public void encode(EncodedSequence encodedSequence) {
		encodedSequence.addInt(vectorpos);
		byte[] namebytes = new byte[Record.L_NAME];
		System.arraycopy(name.getBytes(), 0, namebytes, 0, name.getBytes().length);
		encodedSequence.add(namebytes);
		this.ip.encode(encodedSequence);
		encodedSequence.addLong(t_join);
		encodedSequence.addLong(t_leave);
	}

	
	public static Record parse(EncodedSequence encodedSequence) {
		Record record = new Record();
		record.setVectorpos(encodedSequence.parseInt());
		record.setName(new String(encodedSequence.parseByteArray(L_NAME)).trim());
		IPEndpoint ip = IPEndpoint.parse(encodedSequence);
		record.setIp(ip);
		record.setT_join(encodedSequence.parseLong());
		record.setT_leave(encodedSequence.parseLong());
		return record;
	}
	

	public int getVectorpos() {
		return vectorpos;
	}

	public void setVectorpos(int vectorpos) {
		this.vectorpos = vectorpos;
	}

	public IPEndpoint getIp() {
		return ip;
	}

	public void setIp(IPEndpoint ip) {
		this.ip = ip;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public long getT_join() {
		return t_join;
	}

	public void setT_join(long t_join) {
		this.t_join = t_join;
	}

	public long getT_leave() {
		return t_leave;
	}

	public void setT_leave(long t_leave) {
		this.t_leave = t_leave;
	}

	public byte[] getBytes() {
		EncodedSequence eS = new EncodedSequence(Record.LENGTH);
		eS.addInt(vectorpos);
		byte[] namebytes = new byte[Record.L_NAME];
		System.arraycopy(name.getBytes(), 0, namebytes, 0, name.getBytes().length);
		eS.add(namebytes);
		this.ip.encode(eS);
		eS.addLong(t_join);
		eS.addLong(t_leave);
		return eS.getBytes();
	}
	
	@Override
	public String toString() {
		
		return "["+vectorpos+ " | " +name + " | " + ip.getIpAddress().toString() + ":" + ip.getPort()	+ " | " + t_join + " | " + t_leave + "]";
	}
	
}